import 'package:dio/dio.dart';
import 'package:laba1/data/mappers/spells_mapper.dart';
import 'package:pretty_dio_logger/pretty_dio_logger.dart';

import '../../domain/models/card.dart';
import '../dtos/spells_dto.dart';
import 'api_interface.dart';

class PotterRepository extends ApiInterface {
  static final Dio _dio = Dio()
    ..interceptors.add(PrettyDioLogger(
      requestHeader: true,
      requestBody: true,
    ));

  static const String _baseUrl = 'https://api.potterdb.com';

  @override
  Future<List<CardData>?> loadData(
      {String? q, OnErrorCallback? onError}) async {
    try {
      const String url = '$_baseUrl/v1/spells';

      SpellsDto dto;
      if (q == null) {
        final Response<dynamic> response =
            await _dio.get<Map<dynamic, dynamic>>(url);
        dto = SpellsDto.fromJson(response.data as Map<String, dynamic>);
      } else {
        final Response<dynamic> response1 =
            await _dio.get<Map<dynamic, dynamic>>(
          url,
          queryParameters: q != null ? {'filter[name_cont]': q} : null,
        );
        final Response<dynamic> response2 =
            await _dio.get<Map<dynamic, dynamic>>(
          url,
          queryParameters: q != null ? {'filter[incantation_cont]': q} : null,
        );
        SpellsDto dto1 =
            SpellsDto.fromJson(response1.data as Map<String, dynamic>);
        SpellsDto dto2 =
            SpellsDto.fromJson(response2.data as Map<String, dynamic>);

        List<SpellDataDto> combinedData = [
          ...?dto1.data, // добавляем данные из dto1
          ...?dto2.data // добавляем данные из dto2
        ];
// Удаляем дубликаты по полю id
        Map<String?, SpellDataDto> uniqueSpellsMap = {
          for (var spell in combinedData) spell.id: spell
        };
// Преобразуем обратно в список
        List<SpellDataDto> uniqueSpells = uniqueSpellsMap.values.toList();
// Создаем новый SpellsDto с уникальными данными
        dto = SpellsDto(data: uniqueSpells);
      }
      final List<CardData>? data = dto.data?.map((e) => e.toDomain()).toList();
      return data;
    } on DioException catch (e) {
      onError?.call(e.response?.statusMessage);
      return null;
    }
  }
}
